# How to grab the config file.

package require confile

namespace eval config {
	# confile options
	array set opt {
		err_callback ::config::err
	}

	# configuration file global rules
	array set rules_global {
		fetch.xml {
			type any
			callback ::config::check_exec
			def {curl -fL --connect-timeout 15 --progress-bar --noproxy * -o %f %u}
		}
		fetch.media {
			type any
			callback ::config::check_exec
			def {curl -fLO  --connect-timeout 15 --noproxy *  %u}
		}
	}

	# configuration file feed rules
	array set rules_feed {
		url { type any }
		type {
			type any
			def ""
		}
		category {
			type any
			def ""
		}
		sort {
			type regexp
			regexp {^(increasing|decreasing)$}
			def ""
		}
		treatment {
			type any
			callback ::config::check_exec
			def ""
		}
		fetch {
			type any
			callback ::config::check_exec
			def "-"
		}
	}
}

# fill global conf hash
proc ::config::init {dir} {
	global conf

	# "%s" below is replaced later by [::config::dir_set] with value of its
	# parameter
	set conf(config.file) "%s/config"
	set conf(lock) "%s/.lock"
	set conf(sync) "%s/.sync"
	set conf(media) "%s/media"
	set conf(feeds) "%s/feeds"
	set conf(logs) "%s/logs"
	set conf(hist) "%s/.history.sqlite"

	::config::dir_set $dir
}

proc ::config::dir_set { name } {
	global conf

	foreach i { config.file lock media feeds hist sync logs } {
		regsub -- {%s} $conf($i) $name conf($i)
	}

	if {[catch { file mkdir $conf(media) $conf(feeds) $conf(logs)} cause]} {
		errx 73 "preparing staff in $name: $cause"
	}
}

proc ::config::err { s } {
	global conf
	errx 65 "$conf(config.file): $s"
}

proc ::config::check_exec { cmd } {
	if {$cmd eq "-"} { return 1 }

	if {[catch {which [lindex $cmd 0]} cause]} { return 0 }
	return 1
}

# name - a Tcl file name to eval
#
# updates Cfg array; fills Podcast array
proc ::config::load { name Cfg Podcast } {
    global ::config::opt ::config::rules_global ::config::rules_feed
    upvar $Cfg cfg
    upvar $Podcast podcast

    # eval the configuration file; lame
    if {[catch { source $name } cause]} {
	errx 69 "config: Tcl: $cause"
    }

    if {![array size feed]} { err "no feeds" }

    # 1. global options
    ::confile::check_arr opt rules_global tmp
    array set cfg [array get tmp]
    unset tmp

    # 2. feeds list
    foreach i [array names feed] {
	if {[catch { array set tmp $feed($i) } cause]} {
	    err "variable feed($i): $cause"
	}
	# merge additional options from the command line (-M opt)
	if {[info exists cfg(feed.optmerge)]} {
	    array set tmp $cfg(feed.optmerge)
	}

	::confile::check_arr opt rules_feed tmp
	set feed($i) [array get tmp]
	unset tmp
    }

    array set podcast [array get feed]
}
