# Simple but very useful configuration file checker.

namespace eval confile {
	namespace export check_arr
}

# Example:
#
# proc myerr { s } {
#		puts stderr "foobar error: $s"
#		exit 1
# }
#
# array set opt {
#		err_callback myerr
# }
#
# array set rules {
#		foo {
#			type digit
#			min 0
#			max 10
#			def 5
#		}
#
#		zz {
#			comment "a required variable with max len of 10 chars"
#			type any
#			len 10
#		}
#
#		bar {
#			type regexp
#			regexp {^(yes|no)$}
#			def yes
#		}
#
#		foobar {
#			type any
#			callback my_foobar_check
#			def 2007/02/01
#		}
# }
#
# array set user_config {
#		zz sample
#		foo 9
#		bar no
#		foobar 2008/12/12
# }
#
# ::confile::check_arr opt rules user_config

proc ::confile::check_regexp { are value } {
	if {[regexp -- $are $value]} {
		return 1
	}
	return 0
}

proc ::confile::check_digit { value min max } {
	if {![string is digit $value]} { return 0 }
	if {$value < $min || $value > $max} { return 0 }

	return 1
}

# Opt - array name of some options for the user configuration
# Rules - array name of the parser options for the user configuration
# Userconf - array name with visible user configuration
proc ::confile::check_arr {Opt Rules Userconf} {
	upvar $Opt opt
	upvar $Rules rules
	upvar $Userconf userconf

	foreach i [array name userconf] {
		set r 0
		set userconf($i) [string trim $userconf($i)]

		if {[info exists rules($i)]} {
			array set arr $rules($i)
#			parray arr
			switch -- $arr(type) {
				any {
					if {[info exists arr(len)]
						&& [string length $userconf($i)] > $arr(len)} {
						$opt(err_callback) "variable \"${Userconf}($i)\": too big (> $arr(len) characters)"
					}
					if {[info exists arr(callback)]} {
						# execute callback to check variable value
						set r [$arr(callback) $userconf($i)]
					} else {
						set r 1
					}
				}
				regexp {
					if {[info exists arr(len)]
						&& [string length $userconf($i)] > $arr(len)} {
						$opt(err_callback) "variable \"${Userconf}($i)\": too big (> $arr(len) characters)"
					}
					if {[info exists arr(regexp)]} {
						set r [check_regexp $arr(regexp) $userconf($i)]
					} else {
						error "missing regexp value for confile rule ${Rules}($i)"
					}
				}
				digit {
					if {[info exists arr(min)] && [info exists arr(max)]} {
						set r [check_digit $userconf($i) $arr(min) $arr(max)]
					} else {
						error "missing max or min values for confile rule ${Rules}($i)"
					}
				}
				default {
					error "invalid type $arr(type) in confile rule ${Rules}($i)"
				}
			}
		} else {
			$opt(err_callback) "unknown variable \"${Userconf}($i)\""
		}

		if {!$r} {
			$opt(err_callback) "variable \"${Userconf}($i)\": invalid value \"[string range $userconf($i) 0 16]\""
		}
		unset arr
	}

	foreach i [array name rules] {
		array set arr $rules($i)
		if {![info exists userconf($i)]} {
			if {![info exists arr(def)]} {
				$opt(err_callback) "you must set variable \"${Userconf}($i)\""
			} else {
				set userconf($i) $arr(def)
			}
		}
		unset arr
	}
}
