# Download podcast files.

package require myutils
package require treatment

namespace eval ::download {
	variable treatment
}

# $Feed_info -- name of an array of a feed option from the configuration file.
#
# $hist_check -- callback to a checker which return 1 if a particular
#                file has been downloaded already or 0 if a file is
#                new. checker is a function with 1 argument --
#                url. callback may be "" and than no checks will be
#                performed.
#
# $hist_save -- callback to a function (with 1 argument -- url) which
#               adds url into our history database. callback may be ""
#               and than no saving will be performed.
#
# $down -- callback to a downloader function with 2 arguments:
#          a feed name and a url
#
# returns a list of statistics of downloaded files:
# { { enclosures downloaded failed ignored } { . . . .} }
proc ::download::populate_files { fdct Feed_info hist_check hist_save down } {
	upvar $Feed_info feed_info
	global conf

	# Sort of leave it unsorted? -- that's the question. If a feed
	# generator makes an error with date in the middle of feed entries,
	# than the entry with invalid date will be like a lost sheep and we
	# all will become flustered. So? Let's take the input unsorted.

	# Btw, categories in a feed meta are useless for us: or you are
	# listening a podcast and selecting nice entries in it or you are
	# removing the boring podcast and forgetting about it.

	veputs 2 [format "%s: treatment: %d\n" \
				  [file tail $feed_info(name)] \
				  [expr {$feed_info(treatment) == "" ? 0 : 1}] ]

	set left $conf(entries)
	set stat ""

	set items [dict get $fdct entries]
	set total [dict size $items]
	foreach i [dict keys $items] {
		if {$conf(entries) > 0} {
			if {$left <= 0 } break
			incr left -1
		}

		# current/left/total
		::myutils::veputs 1 "Entry [expr $i + 1]/[expr {$conf(entries) > 0 ? $left : "-"}]/$total:\n"

		set enclosure [dict get $items $i enclosure]
		set category [dict get $items $i category]
		set curstat [list [llength $enclosure] 0 0 0]
		if {[llength $enclosure] == 0} {
			warnx "wtf? entry #[expr $i+1] hasn't any enclosures!"
			continue
		}

#		puts $enclosure
		set encl_total [llength $enclosure]
		set encl_cur 0
		foreach e $enclosure {
			incr encl_cur
			::myutils::veputs 1 "* File $encl_cur/$encl_total: "

			# filter by media type
			set media_type [lindex $e 0]
			if {![::download::filter $media_type $feed_info(type)]} {
				lset curstat 3 [expr [lindex $curstat 3] + 1]
				::myutils::veputs 1 "$media_type is not welcome, ignored\n"
				continue
			}

			# filter by category
			if {![::download::filter $category $feed_info(category)]} {
				lset curstat 3 [expr [lindex $curstat 3] + 1]
				::myutils::veputs 1 "unintresting category, ignored\n"
				continue
			}

			set url [lindex $e 2]
			if {$hist_check != "" && [$hist_check $url]} {
				# this url we have seen already
				lset curstat 3 [expr [lindex $curstat 3] + 1]
				::myutils::veputs 1 "probably has been already downloaded\n"
				continue
			} else {
				::myutils::veputs 1 "new!\n"
			}

			set snapshot [::treatment::dir_snapshot $feed_info(name)]
			set r [$down feed_info $url]
			set p 1
			if {!$r} {
				# oops, cannot get the file, try again next time when
				# this program will be executed
				incr p
			} else {
				# save this url in our database
				if {$hist_save != ""} { $hist_save $url [clock seconds] }
				# add created file to the future treatment operation
				::treatment::add \
					[::treatment::dir_diff $feed_info(name) $snapshot] \
					[::treatment::id3v1 feed_info $fdct $i]
			}
			lset curstat $p [expr [lindex $curstat $p] + 1]

			if {$conf(signal)} break
		}

		if {$conf(signal)} break
		lappend stat $curstat
	}

	return $stat
}

# return 1 if $are match $data or 0 otherwise
proc ::download::filter { data exp } {
	if {$exp == ""} { return 1 }

	foreach i [split $exp ,] {
		set q [string trim $i]
		if {$q != ""} { lappend t $q }
	}

	::myutils::veputs 2 "($data, search for: $t) "
	if {[info exists t]} {
		foreach i $t {
			foreach j $data {
				if {[string match -nocase "*$i*" $j]} { return 1 }
			}
		}
	}

	return 0
}

# Just creates a proper directories and empty file names.
proc ::download::fetch_test { Feed_info url } {
	global conf
	upvar $Feed_info feed_info

	set dir_old [pwd]
	file mkdir $feed_info(name)
	cd $feed_info(name)

#	puts url=$url
	set name [lindex [split $url /] end]
	close [open $name {WRONLY CREAT TRUNC}]
	if {$conf(verbose)} { puts "$name ................................... OK" }

	cd $dir_old
	return 1
}

# for "-a" command line option
proc ::download::fetch_bibb { Feed_info url } {
	veputs 1 "(memorized, but not actually downloaded)\n"
	return 1
}

# for "-C" command line option
proc ::download::fetch_nothing { Feed_info url } {
	veputs 1 "(doing nothing)\n"
	return 0
}

# Return 1 if file was downloaded or 0 if not
proc ::download::fetch { Feed_info url } {
	global conf
	upvar $Feed_info feed_info

	if {[catch {file mkdir $feed_info(name)} cause]} {
		warnx $cause
		return 0
	}

	set dir_old [pwd]
	cd $feed_info(name)
	set r 1

	if {$conf(verbose)} {
		set redfi stdout;		# redirect file
	} else {
		if {[catch {open /dev/null WRONLY} redfi]} {
			errx 69 $redfi
		}
	}

	# choose a fetcher program
	set wget [expr {$feed_info(fetch) ne "-" ? $feed_info(fetch) : $conf(fetch.media)}]

	array set arr [list "u" $url]
	if {[catch {
		exec -- {*}[config_str_format $wget arr] >&@ $redfi
	} cause]} {
		warnx "cannot download: $cause"
		set r 0
	}

	cd $dir_old
	return $r
}

# Return 1 if xml feed was downloaded or 0 if not
proc ::download::feed_get { name url } {
	global conf

	set r 1

	if {$conf(verbose)} {
		set redfi stdout;		# redirect file
	} else {
		if {[catch {open /dev/null WRONLY} redfi]} {
			errx 69 $redfi
		}
	}

	array set arr [list "f" "$conf(feeds)/$name" "u" $url]

	# *** Attention, please, boys and girls ***
	# Make sure that cursor on the terminal is on the first column before
	# executing FreeBSD fetch(1).
	if {[catch {
		exec -- {*}[config_str_format $conf(fetch.xml) arr] >&@ $redfi
	} cause]} {
		warnx "cannot download data for $name: $cause"
		set r 0
	}

	return $r
}

# ---------------------------------------------------------

# if {[llength $argv] != 1} {
# 	errx 1 "usage: $argv0 output_directory < example_feed.xml"
# }

# file mkdir [set conf(outdir) [lindex $argv 0]]
# cd $conf(outdir)

# set root [::feed::init dom_doc "my.xml"]
# puts "feed type: [::feed::get_type $root]"
# if {[catch {feed::parse $root fdct} cause]} {
# 	errx "$::errorInfo"
# }

# array set f {
# 	name "sample podcast"
# 	url "http://foo.bar"
# 	type "audio/mpeg"
# 	category ""
# }

# puts [podcast_populate_files $fdct f "" "" freebsd_downloader 0]

# # free tDOM memory
# $dom_doc delete
