# Check, add, get routines of a history database.

package require sqlite3

# ----------------------------------------------------------------------

namespace eval ::hist {
	# ::hist::dbcmd -- a sqlite command to control the database
}

proc ::hist::open { name } {
	catch {::hist::close}
	sqlite3 ::hist::dbcmd $name
	# check if our database is not a new one
	if {[::hist::dbcmd exists {
		SELECT 1 FROM sqlite_master WHERE type='table' AND tbl_name='h'
	} ]} return;

	::hist::dbcmd eval {
		CREATE TABLE h(url text PRIMARY KEY NOT NULL,
						added integer CHECK(typeof(added)='integer') NOT NULL)
	}
}

proc ::hist::close {} {
	::hist::dbcmd close
}

# raise a Tcl error if we unable to add data
#
# return 1 if something was adder, 0 otherwise
proc ::hist::add { url added } {
	if {![::hist::check $url]} {
		::hist::dbcmd eval {
			INSERT INTO h VALUES($url, $added)
		}
		return 1
	}
	return 0
}

proc ::hist::check { url } {
	return [::hist::dbcmd exists {SELECT 1 FROM h WHERE url=$url}]
}

proc ::hist::last_update {} {
	set t [::hist::dbcmd eval {SELECT added from h ORDER BY added DESC LIMIT 1}]
	if {$t == ""} { return -1 }
	return $t
}

# return a list of {was left} rows or {was -1} if no rows were deleted
#
# O(2*n)
proc ::hist::curtail { t {is_debug 0}} {
	set count [::hist::dbcmd eval {SELECT COUNT(*) FROM h}]
	if {$t < 2 || $t > $count} { return [list $count -1] }

	set d [expr $count-$count/$t]
	if {!$is_debug} {
		::hist::dbcmd eval "DELETE FROM h WHERE rowid<=$d"

		set n 1
		::hist::dbcmd eval {SELECT url FROM h} val {
			::hist::dbcmd eval "UPDATE h SET rowid=$n WHERE url='$val(url)'"
			incr n
		}
		::hist::dbcmd eval "VACUUM"
	}
	return [list $count [expr $count-$d]]
}
