namespace eval ::myutils {
	variable optind 0
	variable optindc 0

	namespace export \
		myexec \
		errx \
		warnx \
		gerrx \
		gwarnx \
		veputs \
		wlist \
		xrdb_get_values \
		xrdb_get_option \
		gen_uniqstr \
		open_tmp \
		which \
		getopt \
		highlight_text \
		config_str_format \
		progress_bar_do \
		progress_bar_end
}

proc ::myutils::myexec { args } {
	set fd [open "| $args 2>@stderr"]
	chan configure $fd -buffering none
	while {[gets $fd line] >= 0} {
		puts $line
	}
	close $fd
}

proc ::myutils::errx {exit_code s} {
	global pref
	set t [expr {[info exists pref(prog.name)] ? $pref(prog.name) : "noname" }]
	puts stderr "$t error: $s"
	if {$exit_code > 0} {
		exit $exit_code
	}
}

proc ::myutils::gerrx { ecode str } {
	global pref
	set t [expr {[info exists pref(prog.name)] ? $pref(prog.name) : "noname" }]
	tk_messageBox -type ok -icon error \
		-title "$pref(prog.name) error" \
		-message $str
	if {$ecode > 0 } { exit $ecode }
}

proc ::myutils::gwarnx { str } {
	global pref
	set t [expr {[info exists pref(prog.name)] ? $pref(prog.name) : "noname" }]
	tk_messageBox -type ok -icon warning \
		-title "$pref(prog.name) warning" \
		-message $str
}

proc ::myutils::warnx {s} {
	global pref
	set t [expr {[info exists pref(prog.name)] ? $pref(prog.name) : "noname" }]
	puts stderr "$t warning: $s"
}

# verbose puts
proc ::myutils::veputs { level s } {
	global conf

	if {$level <= $conf(verbose)} {
		puts -nonewline $s
	}
}

# return a list of all windows in a widget hierarchy
proc ::myutils::wlist {{w .}} {
	set list [list $w]
	foreach i [winfo children $w] {
		set list [concat $list [wlist $i]]
	}
	return $list
}

# Helps user to override startup value stored in -textvariable or
# -variable attribute
proc ::myutils::xrdb_get_values {} {
	global gui

	# make `widget_lvar' array
	foreach w [wlist] {
		foreach i {variable textvariable} {
			if {![catch {$w cget -$i} lval]} {
				if {$lval != ""} { set widget_lvar($w) $lval }
			}
		}
	}

	# for standard widgets set specific X resource name
	foreach i [array names widget_lvar] {
#		puts [winfo class $i]
		switch -regexp -- [winfo class $i] {
			T?Button { set widget_xrdb($i) text }
			T?Checkbutton { set widget_xrdb($i) is_checked }
			T?Label { set widget_xrdb($i) text }
			T?Spinbox { set widget_xrdb($i) spin }
			T?Entry { set widget_xrdb($i) text }
		}
	}
#	parray widget_lvar
#	parray widget_xrdb

	foreach i [array names widget_xrdb] {
		set value [option get $i $widget_xrdb($i) ""]
		if {$value != ""} {
			set $widget_lvar($i) $value
			veputs 1 "$widget_lvar($i) has been set to ``$value''\n"
		}
	}
}

# w - full widget path
# opt - option
# def - default value if opt == ""
proc ::myutils::xrdb_get_option { w opt def } {
	set value [option get $w $opt ""]
	if {$value != ""} {	return $value }
	return $def
}

proc ::myutils::gen_uniqstr { length } {
	set str 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ
	for {set i 0} {$i<$length} {incr i} {
		append uniqstr \
			[string index $str [expr {int(rand()*[string length $str])}]]
	}

	return $uniqstr
}

proc ::myutils::open_tmp { Filename mode prefix } {
	upvar $Filename tmp_file

	set ok 0
	for {set n 0} {$n < 10} { incr n } {
		set tmp_file $prefix.[gen_uniqstr 12]
		if {![file exists $tmp_file]} {
			set ok 1
			break
		}
	}

	if {!$ok} {	error "cannot generate temporally file name; try again" }

	set cmd [list open $tmp_file $mode]
	if {[catch {eval $cmd} fd]} { error $fd }

	return $fd
}

# (ripped from TkCon)
# tells you where a command is found
# cmd - command name
# Returns: where command is found (internal / external / unknown)
proc ::myutils::which {cmd} {
    if {[string compare {} [info commands $cmd]] || \
			([auto_load $cmd] && [string compare {} [info commands $cmd]])} {
		if {[lsearch -exact [interp aliases] $cmd] > -1} {
			return "$cmd: aliased to [alias $cmd]"
		} elseif {[string compare {} [info procs $cmd]]} {
			return "$cmd: internal proc"
		} else {
			return "$cmd: internal command"
		}
    } elseif {[string compare {} [auto_execok $cmd]]} {
		return [auto_execok $cmd]
    } else {
		return -code error "$cmd: unknown command"
    }
}

# See <http://www.waxandwane.com/toolbox/tclGetOpts/>
# for full source & documentation of this function.
proc ::myutils::getopt { argslist optstring optret argret } {
	variable optind
	variable optindc
	upvar $optret retvar
	upvar $argret optarg

	# default settings for a normal return
	set optarg ""
	set retvar ""
	set retval 0

	# check if we're past the end of the args list
	if { $optind < [ llength $argslist ] } then {

		# if we got -- or an option that doesn't begin with -, return (skipping
		# the --).  otherwise process the option arg.
		switch -glob -- [ set arg [ lindex $argslist $optind ]] {
			"--" {
				incr optind
			}

			"-*" {
				if { $optindc < 1 } then {
					set optindc 1
				}

				set opt [ string index $arg $optindc ]

				if { [ incr optindc ] == [ string length $arg ] } then {
					set arg [ lindex $argslist [ incr optind ]]
					set optindc 0
				}

				if { [ string match "*$opt*" $optstring ] } then {
					set retvar $opt
					set retval 1
					if { [ string match "*$opt:*" $optstring ] } then {
						if { $optind < [ llength $argslist ] } then {
							set optarg [ string range $arg $optindc end ]
							incr optind
							set optindc 0
						} else {
							set optarg "option requires an argument -- $opt"
							set retvar $optarg
							set retval -1
						}
					}
				} else {
					set optarg "illegal option -- $opt"
					set retvar $optarg
					set retval -1
				}
			}
		}
	}

	return $retval
}

# <b> - bold
# <i> - green
# <h3> - (current size) + bold + brown
# <h2> - (size=+2) + bold + brown
# <h1> - (size=+4) + bold + brown
# highlights URL & e-mail automatically
proc ::myutils::highlight_text { path } {
	array set arec {
		url {(https?|ftp|gopher)://([A-Za-z\d_.=\+?~&%/-]+?)(?::([0-9]+))?([A-Za-z\d_.=\+?~&%/-]+?)}
		email {[A-Za-z\d_.-]+@[A-Za-z\d_.-].[A-Za-z\d_.-]+}
		bold {<b>.*?</b>}
		italic {<i>.*?</i>}
 		h1 {<h1>.*?</h1>}
 		h2 {<h2>.*?</h2>}
 		h3 {<h3>.*?</h3>}
	}
	array set markup {
		url {0 0}
		email {0 0}
		bold {3 4}
		italic {3 4}
		h1 {4 5}
		h2 {4 5}
		h3 {4 5}
	}

	# make tags; every tag has name equal `arec' index + "_tag" suffix
	$path tag configure url_tag -foreground blue -underline 1
	$path tag configure email_tag -foreground #9a32cd
	$path tag configure bold_tag \
		-font "[font actual [$path cget -font]] -weight bold"
	$path tag configure italic_tag -foreground #008b00
	$path tag configure h1_tag -foreground #990000 \
		-font "[font actual [$path cget -font]] -weight bold \
-size [expr [font actual [$path cget -font] -size]+5]"
	$path tag configure h2_tag -foreground #990000 \
		-font "[font actual [$path cget -font]] -weight bold \
-size [expr [font actual [$path cget -font] -size]+1]"
	$path tag configure h3_tag -foreground #990000 \
		-font "[font actual [$path cget -font]] -weight bold"

    #" emacs fix
	foreach i [array names arec] {
		set start 1.0
		while {[set first [$path search -regexp -count num -- $arec($i) $start end]] != ""} {
			set last [$path index "$first + $num chars"]
			$path tag add ${i}_tag $first $last

			# delete markup
			set md_1 [lindex $markup($i) 0]
			set md_2 [lindex $markup($i) 1]
			if {$md_1 && $md_2} {
				$path delete $first "$first + $md_1 chars" \
					"$last - $md_2 chars" $last
				set start "$last - $md_2 chars"
			} else { set start $last }
		}
	}
}

# Convert format characters to their values.
#
# s - semi-finished product string
# Format - array {format_char value}
#
# Return resolved string.
#
# FIXME: broken behaviour with "%%" sequences
#        > array set arr "s foo"
#        > config_str_format "%%s/con%%fig" arr
#        %f&o/con%fig
#
#        while must return %s/con%fig
proc ::myutils::config_str_format { s Format } {
	upvar $Format f

	array set a [array get f]
	foreach i [array names a] {
		regsub -all -- {&} $a($i) {\\&} a($i)
		regsub -all -- "%$i" $s [list $a($i)] s
	}
	regsub -all -- {%%} $s % s
	return $s
}

# Draws a "progress bar", then depending on `Per' decides to [puts] or
# not to [puts] a single '*' char.
#
# Arguments:
# scale			length of the bar
# length		end point, i.e. number which is equal to 100%
# i				current position
# Per			list that func reads to determine her last percent;
#				user don't need to worry about it
proc ::myutils::progress_bar_do { scale length i Per } {
	upvar $Per per
	if {![info exists per]} {
		set per 0
		puts -nonewline stderr "0%"
		for {set j 0} {$j<$scale} {incr j} { puts -nonewline stderr "." };
		puts -nonewline stderr "100%\n |"
	}

	set sp [expr 100.0/$scale]
	set p [expr $i/$length.0*100]
	set point [expr round($p/$sp)]
	if {[lindex $per end] != $point} {
		for {set j 0} {$j<$point-[lindex $per end]} {incr j} {
			puts -nonewline stderr *
		}
		lappend per $point
	}
}

# Companion to [progress_bar_do].
# Run it only ONCE after loop with progress_bar_do().
proc ::myutils::progress_bar_end { Per } {
	upvar $Per per
	puts stderr "| done."
	unset per
}
