#!/bin/sh

# This script was written with special guidelines for uraniacast because
# it is intended to run by uraniacast, although it may be used as a
# standalone program as well.
#
# See uraniacast_treatment(7).
#
# Requires: id3lib, exiftool
#
# Test:
#
# $ (export URANIACAST_LOGS=.; export URANIACAST_FEED_CUR=foo.bar; \
#   ./uraniacast_t_mp3tag.sh 01.mp3 \
#   'an artist' 'an album' 'a song' 2013 'a comment')
#
# Then check 'foo.bar/treatments/01.mp3.txt' file.

SCRIPT_PATH=$(dirname $(realpath $0))
. $SCRIPT_PATH/lib/bourne/helper.sh

prerequisites id3tag exiftool || exit 1
[ -z "$1" ] && errx "Usage: ${0##*/} file.mp3 artist album song year comment"

treatment_begin 'mp3 tagging...'

[ "`exiftool -T -mimetype $1 2>/dev/null`" != "audio/mpeg" ] && \
	errx "$1 is not an mp3 file"

log=`treatment_log "$1"` || exit 1
id3tag \
	--artist="$2" \
	--album="$3" \
	--song="$4" \
	--year="$5" \
	--comment="$6" \
	"$1" > "$log" 2>&1

treatment_end
